
#pragma once

// Genode includes
#include <base/log.h>
#include <util/string.h>

// protobuf includes
#include <google/protobuf/message.h>
#include <google/protobuf/util/time_util.h>

// stdcxx includes
#include <vector>

// gapfruit includes
#include <tee_session/tee_session.h>

// generated includes
#include <tee.pb.h>


namespace Tee
{
	using Genode::Hex;
	using Genode::Output;
	using Genode::log;

	using google::protobuf::Timestamp;
	using google::protobuf::Duration;

	using gapfruit::protobuf::tee::Address;
	using gapfruit::protobuf::tee::CounterRequest;
	using gapfruit::protobuf::tee::ExecuteInitRequest;
	using gapfruit::protobuf::tee::ExecuteInitResponse;
	using gapfruit::protobuf::tee::ExecuteRequest;
	using gapfruit::protobuf::tee::ExecuteResponse;
	using gapfruit::protobuf::tee::Hash;
	using gapfruit::protobuf::tee::QuoteRequest;
	using gapfruit::protobuf::tee::Request;
	using gapfruit::protobuf::tee::Signature;
	using gapfruit::protobuf::tee::SignedRequest;

	using Packet_content_ro = const char*;
	using Packet_content_rw = char*;

	struct Deserialize_failed  : Genode::Exception { };
	struct Serialize_failed    : Genode::Exception { };
	struct Transmission_error  : Genode::Exception { };

	using Certificate_string = std::string;
	using Fingerprint_string = std::string;
	using Fingerprints       = std::vector<Fingerprint_string>;
	using Hash_string        = std::string;
	using Nonce_data         = std::string;
	using Request_data       = std::string;
	using Response_data      = std::string;
	using Security_domain    = std::string;
	using Signature_string   = std::string;

	struct Hash_data;
	struct Request_info;

	enum {
		NONCE_SIZE = 16,  // number of bytes to store a nonce
	};

	// Create a Timestamp initialized with the current system time.
	Timestamp timestamp_from_system_time();
}


struct Tee::Hash_data
{
	Hash::Algorithm algorithm;
	Hash_string     value;
};


struct Tee::Request_info
{
	Timestamp           timestamp;
	Session_id          session_id;
	Security_domain     security_domain;
	Certificate_string  certificate;
	Hash_string         hash;
	Signature_string    signature;
};
